<?php

/*
Plugin Name: LDS Linker
Plugin URI: http://www.joeyday.com/tag/lds-linker
Description: Automagically changes any Latter-day Saint Standard Works reference to a hyperlink to the passage at scriptures.lds.org.
Version: 2.1 beta 1
Author: Joey Day
Author URI: http://www.joeyday.com
*/

// Add some default options if they don't already exist.
add_option('ldslinker_url_style', 'direct', 'Style for LDS scripture URLs. Possible values are: direct (Direct lookup), or search (Passage search)');
add_option('ldslinker_include_bible_refs', TRUE, 'Linkify references to the Holy Bible.');
add_option('ldslinker_language', 'en', 'Language to use when linking to LDS scripture passages. Possible values are: de (Deutsch), en (English), es (Espaol), fr (Franais), or it (Italiano).');

// BEGIN ADMIN CONSOLE
if (! function_exists('ldslinker_add_options')) {
    function ldslinker_add_options() {
        if (function_exists('add_options_page')) {
            add_options_page('LDS Linker Options', 'LDS Linker', 9, basename(__FILE__), 'ldslinker_options_subpanel');
        }
    }
}

function ldslinker_options_subpanel() {
    if ( isset($_POST['info_update']) ) {
        update_option('ldslinker_url_style', $_POST['ldslinker_url_style']);
        update_option('ldslinker_include_bible_refs', $_POST['ldslinker_include_bible_refs']);
        ?><div class="updated"><p><strong><?php 
            _e('Options saved.', 'LDS Linker')
        ?></strong></p></div><?php
    } ?>
    <div class="wrap">
        <form method="post">
            <h2>LDS Linker Options</h2>
            <?php // Get values from options table in database.
            $url_style = get_option('ldslinker_url_style');
            $include_bible_refs = get_option('ldslinker_include_bible_refs'); ?>
            <table class="optiontable"> 
                <tr valign="top"> 
                    <th scope="row">URL style:</th> 
                    <td>
                        <label for="ldslinker_url_style"> 
                        <select name="ldslinker_url_style" id="ldslinker_url_style">
                            <option value="direct"<?php if ($url_style == 'direct') echo ' selected="selected"'; ?>>Direct lookup</option>
                            <option value="search"<?php if ($url_style == 'search') echo ' selected="selected"'; ?>>Passage search</option>
                        </select>
                        <br />
                        There are two ways to bring up passages on scriptures.lds.org: a passage search or a direct passage lookup. To see the difference, compare the following two examples.
                        <ul>
                            <li>Passage search: <a href="http://scriptures.lds.org/en/search?search=1+Ne.+3%3A7" target="_new1">1 Nephi 3:7</a></li>
                            <li>Direct lookup: <a href="http://scriptures.lds.org/en/1_ne/3/7#7" target="_new2">1 Nephi 3:7</a></li>
                        </ul>
                    </td>
                </tr>
                <tr valign="top">
                    <th scope="row">Linkify Bible references:</th>
                    <td>
                        <input type="checkbox" name="ldslinker_include_bible_refs" id="ldslinker_include_bible_refs"<?php if ($include_bible_refs) echo ' checked="checked"'; ?> />
                        <br />
                        Check this box if you'd like LDS Linker to linkify Bible passages. If you'd like to use an alternative plugin like <a href="http://dev.wp-plugins.org/wiki/Scripturizer">Scripturizer</a> to link Bible verses, uncheck this option.
                    </td> 
                </tr> 
            </table>
            <div class="submit">
            <input type="submit" name="info_update" value="<?php
                _e('Update options', 'Localization name')
            ?> &raquo;" /></div>
        </form>
    </div><?php
}
// END ADMIN CONSOLE

/**
 * Namespace for LDSLinker.
 */
class LDSLinker {

    /**
     * Hyperlink Latter-day Saint Standard Works references in text.
     *
     * @param data String containing the text.
     * @param callback A function object to be used to replace scripture
     *     references with hyperlinks.
     * @return Linkified text.
     */
    function addLink($data, $callback) {
    
        // A volume number between 1 and 4 (for books like 1 Nephi or 3 John)
        $volumes = '[1-4]';

        // Names of books either written out in full or abbreviated using the
        // standard abbreviations found at: http://scriptures.lds.org/en/helps/abbrvtns
        $books = '';
        if ( get_option('ldslinker_include_bible_refs') ) {
            $books .= 'Genesis|Gen\.?|Exodus|Ex\.?|Leviticus|Lev\.?|'.
                'Numbers|Num\.?|Deuteronomy|Deut\.?|'.
                'Joshua|Josh?\.?|Judges|Judg\.?|Ruth|Samuel|Sam\.?|'.
                'Kings|Kgs\.?|Chronicles|Chr\.?|Ezra|'.
                'Nehemiah|Neh\.?|Esther|Esth\.?|Job|'.
                'Psalms?|Ps\.?|Proverbs?|Prov\.?|Ecclesiastes|Eccl\.?|'.
                'Song of Solomon|Song\.?|Isaiah|Isa\.?|Jeremiah|Jer\.?|'.
                'Lamentations|Lam\.?|Ezekiel|Ezek\.?|Daniel|'.
                'Dan\.?|Hosea|Joel|Amos|Obadiah|Obad\.?|Jonah|'.
                'Micah|Nahum|Habakkuk|Hab\.?|Zephaniah|'.
                'Zeph\.?|Haggai|Hag\.?|Zechariah|Zech\.?|Malachi|Mal\.?|'.
                'Matthew|Matt\.?|Mark|Luke|John|Jn\.?|'.
                'Acts|Romans|Rom\.?|Corinthians|Cor\.?|'.
                'Galatians|Gal\.?|Ephesians|Eph\.?|'.
                'Philippians|Philip\.?|Colossians|Col\.?|Thessalonians|'.
                'Thes\.?|Timothy|Tim\.?|Titus|Philemon|Philem\.?|'.
                'Hebrews|Heb\.?|James|Peter|Pet\.?|Jude|'.
                'Revelation|Rev\.?|';
        }

        $books .= 'Nephi|Ne\.?|Jacob|Enos|Jarom|Omni|Words\sof\sMormon|W\.?\sof\sM\.?|'.
            'Mosiah|Alma|Helaman|Hel\.?|Mormon|Morm\.?|Ether|Moroni|Moro\.?|'.
            'Doctrine\sand\sCovenants|D\s?&\s?C|D\s?&amp;\s?C|D\s?&\#0?38;\s?C|'.
            'Joseph\sSmith\sMatthew|Joseph\sSmith\s?&mdash;\s?Matthew|Joseph\sSmith\s?-\s?Matthew|'.
            'Joseph\sSmith\s?&\#8212;\s?Matthew|Joseph\sSmith\s?\s?Matthew|JS-M|'.
            'Joseph\sSmith\sHistory|Joseph\sSmith\s?&mdash;\s?History|Joseph\sSmith\s?-\s?History|'.
            'Joseph\sSmith\s?&\#8212;\s?History|Joseph\sSmith\s?\s?History|JS-H|'.
            'Abraham|Abr\.?|Moses|Official\sDeclaration|O\.?\s?D\.?|Articles\sof\sFaith|A\.?\sof\sF\.?';

        // The chapter: 1 or more digits followed possibly by a colon and a space.
        $chapter = '\d+(?::\s)?:?';

        // The verse(s): 1 or more digits followed possibly by a hyphen or an en dash and 1 or more digits.
        $verses  = '\d+(?:\s?(-||&ndash;)\s?\d+)?';

        // String all the above regexes together to form the entire passage regex.
        // Notice that $verses is used recursively to catch comma separated verse
        // ranges in the same chapter, such as 1 Nephi 11:1-2, 15.
        return preg_replace_callback("/(?:($volumes)\s)?($books)(?=\s)(?:\s($chapter))($verses(?:\s?,\s?$verses)*)?/mx", $callback, $data);

    }
    
    /**
     * Callback function for replacing scripture references in the text with
     * HTML hyperlinks.
     *
     * @param match Regex matching result array.
     * @return A shiny new hyperlink.
     */
    function addLinkCallback($match) {

        $vol = $match[1];
        $bok = $match[2];
        $chp = $match[3];
        $ver = $match[4];

        // Save a cleaned up version of the complete passage before we mangle the variables.
        $psg = xmlentities(($vol ? "$vol " : "").$bok.($chp ? " $chp" : "").($ver ? "$ver" : ""));

        // Prepare the first part of the link. This is the same whether we're using
        // passage search or direct lookup.
        $link = 'http://scriptures.lds.org/'. get_option('ldslinker_language') . '/';

        // This allows linking directly to an Article of Faith without including the
        // chapter number.
        if ((preg_match('/Articles\sof\sFaith/', $bok) || preg_match('/A\.?\sof\sF\.?/', $bok)) && !$ver) {
            $ver = $chp;
            $chp = '1:';
        }

        // Find out whether the user has selected direct lookup or passage search.
        // Do the following if they've selected direct lookup.
        if ( get_option('ldslinker_url_style') == 'direct' ) {

            // If there's a volume number, tack an underscore to the end of it.
            if ($vol) $vol .= '_';
        
            // Trim whitespace and possible em dashes, dots, and ampersands from book
            // and translate all to lowercase.
            $bok = strtolower(preg_replace(array('/&mdash;/', '/&\#0?151;/', '//', '/-/', '/\s/', '/\./', '/&(?:amp;)?(?:\#0?38;)?/'),
                                array('', '', '', '', '', '', ''),
                                $bok));            

            // If the book isn't abbreviated, we need to abbreviate it before we tack
            // it onto the link string. We start by creating an array of book names
            // and their abbreviations.
            $abbr = array (
                'genesis' => 'gen',         'exodus' => 'ex',                'leviticus' => 'lev',
                'numbers' => 'num',         'deuteronomy' => 'deut',         'joshua' => 'josh',
                'judges' => 'judg',         'samuel' => 'sam',               'kings' => 'kgs',
                'chronicles' => 'chr',      'nehemiah' => 'neh',             'esther' => 'esth',
                'psalms' => 'ps',           'proverbs' => 'prov',            'ecclesiastes' => 'eccl',
                'songofsolomon' => 'song',  'isaiah' => 'isa',               'jeremiah' => 'jer',
                'lamentations' => 'lam',    'ezekiel' => 'ezek',             'daniel' => 'dan',
                'obadiah' => 'obad',        'habakkuk' => 'hab',             'zephaniah' => 'zeph',
                'haggai' => 'hag',          'zechariah' => 'zech',           'malachi' => 'mal',
                'matthew' => 'matt',        'romans' => 'rom',               'corinthians' => 'cor',
                'galatians' => 'gal',       'ephesians' => 'eph',            'philippians' => 'philip',
                'colossians' => 'col',      'thessalonians' => 'thes',       'timothy' => 'tim',
                'philemon' => 'philem',     'hebrews' => 'heb',              'peter' => 'pet',
                'revelation' => 'rev',      'nephi' => 'ne',                 'wordsofmormon' => 'w_of_m',
                'wofm' => 'w_of_m',         'helaman' => 'hel',              'mormon' => 'morm',
                'moroni' => 'moro',         'doctrineandcovenants' => 'dc',  'josephsmithmatthew' => 'js_m',
                'jsm' => 'js_m',            'josephsmithhistory' => 'js_h',  'jsh' => 'js_h',
                'abraham' => 'abr',         'officialdeclaration' => 'od',   'articlesoffaith' => 'a_of_f',
                'aoff' => 'a_of_f'
            );

            // If the book matches one of the full length book names from the array,
            // translate it into the appropriate abbreviation.
            if ($abbr[$bok]) $bok = $abbr[$bok];
        
            // John is an odd case here. The Gospel of John is not abbreviated, but
            // the three epistles of John are abbreviated with 'jn'. In other words, if
            // there's not a volume number, we need to leave 'john' alone, but if there is
            // a volume number we need to abbreviate it.
            if ($vol && $bok == 'john') $bok = 'jn';

            // Trim possible colon and whitespace from end of chapter.
            $chp = preg_replace(array('/:/', '/\s/'), array('', ''), $chp);

            // URL-encode colons, commas, and en dashes and trim superfluous whitespace from verse(s).
            $ver = preg_replace(array('/:/', '/,/', '//', '/&ndash;/', '/\s*/'),
                                array('%3A', '%2C', '-', '-', ''),
                                $ver);

            // Figure out which verse is first in the set of verses so we can use it as
            // an anchor in the URL.
            preg_match('/\d+/', $ver, $result);
            $anc = $result[0];

            // Smoosh all this together to produce the final URL.
            $link .= $vol.$bok.($chp ? "/$chp" : "").($ver ? "/$ver" : "").($anc ? "#$anc" : "");

        }
        
        // Do the following if the user has selected passage search.
        else {
        
            // For some reason the official declarations can't be found using a passage 
            // search, so we have to do a direct lookup. Weird.
            if (preg_match('/Official\sDeclaration/', $bok) || preg_match('/O\.?\s?D\.?/', $bok)) {
                $link .= 'od/'.$chp;
            }

            // For everything else, we URL-encode the passage and tack it
            // to the end of the link in the form of a search query.
            else {

                // Change possible em dash or hyphen to space, change possible whitespace to +,
                // trim possible dots, URL-encode possible ampersand from book.
                $bok = preg_replace(array('/&mdash;/', '/&\#0?151;/', '//', '/Smith\s?-\s?/', '/\s/', '/\./', '/&(?:amp;)?(?:\#0?38;)?/'),
                                    array('+', '+', '+', 'Smith+', '+', '', '%26'),
                                    $bok);
                                    
                // Trim possible whitespace from end of chapter.
                $chp = preg_replace('/\s$/', '', $chp);
                
                // URL-encode colons, commas, and en dashes and trim superfluous whitespace from verse(s).
                $ver = preg_replace(array('/:/', '/,/', '//', '/&ndash;/', '/&#0?150;/', '/\s*/'),
                                    array('%3A', '%2C', '-', '-', '-', ''),
                                    $ver);
        
                // Tack the cleaned up volume, book, chapter and verse(s) to the link
                // in the form of a search query.
                $link .= 'search?search='.($vol ? "$vol+" : "").$bok.($chp ? "+$chp" : "").$ver;
                
            }
        }

        // Pass back the shiny new hyperlink.   
        return "<a href=\"$link\" title=\"LDS Scriptures Internet Edition: $psg\">$psg</a>";
    }

    /**
     * Linkify a text string.
     *
     * @param text Original text to be linkified.
     * @return Linkified text.
     */
    function ldslinkify($text) {

        $callback = create_function('$match', "return LDSLinker::addLinkCallback(".'$match'.");");

        // We don't want to linkify passages that are already hyperlinked
        // or surrounded in <pre> or <code> tags, so we strip those out by
        // splitting the string each time we encounter one of those.
        $text = preg_split("/(<a href.*?<\/a>|<code.*?<\/code>|<pre.*?<\/pre>|<.+?>)/ms", $text, -1, PREG_SPLIT_DELIM_CAPTURE);

        // Build a new array composed of text chunks by parsing each element of
        // the array we just built above. This is where we call the code that
        // actually creates the scripture passage links.
        $newtext = array();
        foreach ($text as $chunk) {
            if (!preg_match('/<.*>/', $chunk)) {
                $chunk = LDSLinker::addLink($chunk, $callback);
            }
            $newtext[] = $chunk;
        }

        // Attach all the array elements back together into a single
        // string and return the new string.
        return implode('', $newtext);
    }

}

/**
 * Function to encode entities. Works like htmlentities but a little less sloppy.
 * I lifted this from an anonymous comment at http://www.php.net/htmlentities.
 * I mostly needed it so D&C references wouldn't show up as D&amp;C.
 *
 * @param string String containing text to me encoded
 * @param quote_style Flag to determine style of quotes
 * @return Encoded text
 */
function xmlentities($string, $quote_style=ENT_QUOTES) {
   static $trans;
   if (!isset($trans)) {
       $trans = get_html_translation_table(HTML_ENTITIES, $quote_style);
       foreach ($trans as $key => $value)
           $trans[$key] = '&#'.ord($key).';';  // don't translate the '&' in case it is part of &xxx;
       $trans[chr(38)] = '&';
   }
   // after the initial translation, _do_ map standalone '&' into '&#38;'
   return preg_replace("/&(?![A-Za-z]{0,4}\w{2,3};|#[0-9]{2,3};)/","&#38;" , strtr($string, $trans));
}

// If 'add_filter' function exists, then we assume that we are initialised as
// a WordPress plugin. We will try to create a new global scope function, and
// add it to filter post and comment content.
if (function_exists('add_filter')) {
    function __ldslinkify($content) {
        return LDSLinker::ldslinkify($content);
    }

    add_filter('the_content', '__ldslinkify', 10);
    add_filter('comment_text', '__ldslinkify', 10);
}

// Load the Options page into the Admin Console
add_action('admin_menu', 'ldslinker_add_options');